----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:19:08 03/21/2011 
-- Design Name: 
-- Module Name:    DDR2_Control_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DDR2_Control_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		reset_in : in std_logic;
		clk_in : in std_logic;
		clk90_in : in std_logic;		
		init_done : in std_logic;
		command_register : out std_logic_vector(2 downto 0);
		input_adress : out std_logic_vector(24 downto 0);
		input_data : out std_logic_vector(31 downto 0);
		output_data : in std_logic_vector(31 downto 0);
		cmd_ack : in std_logic;
		data_valid : in std_logic;
		burst_done : out std_logic;
		auto_ref_req : in std_logic;
		btn_in : in std_logic_vector(5 downto 0);
		status_out : out std_logic_vector(7 downto 0)
	);

end DDR2_Control_VHDL;

architecture Verhalten of DDR2_Control_VHDL is

	--------------------------------------------
	-- Einbinden einer Componente
	--------------------------------------------
	COMPONENT DDR2_Write_VHDL
	PORT (
		reset_in : in std_logic;
		clk_in : in std_logic;	
		w_command_register : out std_logic_vector(2 downto 0);
		w_cmd_ack : in std_logic;
		w_burst_done : out std_logic;
		write_en : in std_logic;
		write_busy : out std_logic;		
		write_status_out : out std_logic_vector(7 downto 0)	
	);
	END COMPONENT DDR2_Write_VHDL;
	
	--------------------------------------------
	-- Einbinden einer Componente
	--------------------------------------------
	COMPONENT DDR2_Read_VHDL
	PORT (
		reset_in : in std_logic;
		clk_in : in std_logic;
		r_command_register : out std_logic_vector(2 downto 0);
		r_cmd_ack : in std_logic;
		r_burst_done : out std_logic;	
		r_data_valid : in std_logic;
		r_in_data : in std_logic_vector(31 downto 0);
		r_out_data : out std_logic_vector(31 downto 0);		
		read_en : in std_logic;
		read_busy : out std_logic;
		read_status_out : out std_logic_vector(7 downto 0)
	);
	END COMPONENT DDR2_Read_VHDL;	

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------

--	constant init_pause : integer := 33500; -- pause 250us  
	constant init_pause : integer := 133000000; -- pause 1s	
	
	type STATE_M_TYPE is (
		M1_START_UP,
		M2_WAIT_4_DONE,
		M3_NOP,
		M4_WRITE_INIT,
		M5_WRITE,
		M6_WRITING,
		M7_WRITE_OK,
		M8_READ_INIT,
		M9_READ,
		M10_READING,
		M11_READ_OK
	);
	signal STATE_M : STATE_M_TYPE := M1_START_UP;	
	
	signal v_counter :  natural range 0 to init_pause := init_pause;
	signal v_write_en : std_logic:='0'; -- '1'=chip-select
	signal v_read_en : std_logic:='0'; -- '1'=chip-select
	signal v_write_busy : std_logic; -- '1'=belegt, '0'=frei
	signal v_read_busy : std_logic; -- '1'=belegt, '0'=frei
	signal v_main_status : std_logic_vector(7 downto 0):= (others => '0');
	signal v_write_status : std_logic_vector(7 downto 0):= (others => '0');	
	signal v_read_status : std_logic_vector(7 downto 0):= (others => '0');	
	signal v_write_command_register : std_logic_vector(2 downto 0):= (others => '0');
	signal v_read_command_register : std_logic_vector(2 downto 0):= (others => '0');
	signal v_write_burst_done : std_logic;
	signal v_read_burst_done : std_logic;
	signal v_read_data : std_logic_vector(31 downto 0):= (others => '0');
	
	-- diese Variable wird ins RAM geschrieben
	signal v_data_wort : std_logic_vector(31 downto 0):="00000001000000110000011100001111"; 		
	
begin

	--------------------------------------------------
	-- Instatz einer Componente erzeugen und verbinden
	--------------------------------------------------
	INST_DDR2_Write_VHDL : DDR2_Write_VHDL
	PORT MAP (
		reset_in => reset_in,
		clk_in => clk_in,
		w_command_register => v_write_command_register,
		w_cmd_ack => cmd_ack,
		w_burst_done => v_write_burst_done,
		write_en => v_write_en,
		write_busy => v_write_busy,		
		write_status_out => v_write_status	
	);
	
	--------------------------------------------------
	-- Instatz einer Componente erzeugen und verbinden
	--------------------------------------------------
	INST_DDR2_Read_VHDL : DDR2_Read_VHDL
	PORT MAP (
		reset_in => reset_in,
		clk_in => clk_in,
		r_command_register => v_read_command_register,
		r_cmd_ack => cmd_ack,
		r_burst_done => v_read_burst_done,	
		r_data_valid => data_valid,
		r_in_data => output_data,
		r_out_data => v_read_data,
		read_en => v_read_en,
		read_busy => v_read_busy,
		read_status_out => v_read_status		
	);	

	-----------------------------------------
	-- State-Machine zum init des RAM
	-- und starten von einem Read oder Write
	-- per Taste (West oder East)
	-----------------------------------------	
	P_State_Main : process(clk_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_M <= M1_START_UP;
			v_write_en <= '0';
			v_read_en <= '0';
			command_register <= "000"; -- NOP
			v_counter <= init_pause;
			v_data_wort <= "00000001000000110000011100001111";
		elsif falling_edge(clk_in) then
			case STATE_M is
				when M1_START_UP =>
					-- warte 250us nach Reset bis RAM bereit
					if v_counter = 0 then					
						-- nach 250us INIT-Kommando (fr einen Clock) anlegen
						STATE_M <= M2_WAIT_4_DONE;
						command_register <= "010";	-- INIT-CMD	
					else 
						command_register <= "000"; -- NOP
						v_counter <= v_counter - 1;
					end if;	
				when M2_WAIT_4_DONE =>
					-- warte auf Init-Done-Signal vom RAM
					command_register <= "000"; -- NOP
					if (init_done = '1') then
						-- das RAM ist jetzt bereit
						STATE_M <= M3_NOP;						
					end if;					
				when M3_NOP =>
					-- warte auf Taste fuer READ oder WRITE
					v_write_en <= '0';
					v_read_en <= '0';					
					if btn_in(3) = '1' and v_write_busy = '0' and auto_ref_req = '0' then
						-- button = west
						-- write starten (nur wenn nicht busy und kein refresh-zyklus)
						STATE_M <= M4_WRITE_INIT;
					elsif btn_in(0) = '1' and v_read_busy = '0' and auto_ref_req = '0' then
					   -- button = east
						-- read starten (nur wenn nicht busy und kein refresh-zyklus)
						STATE_M <= M8_READ_INIT;
					end if;
					
					-- warte auf Taste fuer Data-Up oder Data-Down
					if btn_in(1)='1' then
						-- button = north
						v_data_wort <= v_data_wort + '1';
					elsif btn_in(2)='1' then
						-- button = south
						v_data_wort <= v_data_wort - '1';
					end if;
				when M4_WRITE_INIT =>					
					-- schreiben vorbereiten					
					STATE_M <= M5_WRITE;
					-- adresse und daten anlegen
					input_adress <= "0000000000000000000000000";
					input_data <= v_data_wort;
				when M5_WRITE =>
					-- leitungen von write weiterleiten
					command_register <= v_write_command_register;
					burst_done <= v_write_burst_done;
					-- warten bis zum schreiben bereit
					if v_write_busy = '0' and v_write_en='0' then
						-- daten zum schreiben freigeben
						v_write_en <= '1';
					elsif v_write_busy = '1' and v_write_en='1' then
						-- daten werden geschrieben
						v_write_en <= '0';
						STATE_M <= M6_WRITING;
					end if;
				when M6_WRITING =>					
					-- leitungen von write weiterleiten
					command_register <= v_write_command_register;
					burst_done <= v_write_burst_done;					
					-- warte bis schreiben fertig
					if v_write_busy = '0' then
						STATE_M <= M7_WRITE_OK;
					end if;
				when M7_WRITE_OK =>
					-- warte bis keine Taste
					if btn_in(3) = '0' then
						STATE_M <= M3_NOP;
					end if;
				when M8_READ_INIT =>					
					-- lesen vorbereiten					
					STATE_M <= M9_READ;
					-- adresse anlegen
					input_adress <= "0000000000000000000000000";					
				when M9_READ =>
					-- leitungen von read weiterleiten
					command_register <= v_read_command_register;
					burst_done <= v_read_burst_done;
					-- warten bis zum lesen bereit
					if v_read_busy = '0' and v_read_en='0' then 
						-- daten zum lesen freigeben
						v_read_en <= '1';
					elsif v_read_busy = '1' and v_read_en='1' then
						-- daten werden gelesen
						v_read_en <= '0';						
						STATE_M <= M10_READING;
					end if;
				when M10_READING =>
					-- leitungen von read weiterleiten
					command_register <= v_read_command_register;	
					burst_done <= v_read_burst_done;
					-- warte bis lesen fertig
					if v_read_busy = '0' then
						STATE_M <= M11_READ_OK;
					end if;					
				when M11_READ_OK =>
					-- warte bis keine Taste
					if btn_in(0) = '0' then
						STATE_M <= M3_NOP;
					end if;					
				when others =>
					NULL;
			end case;
		end if;
	end process P_State_Main;	
	
	-----------------------------------------
	-- Bearbeiten von Debug Signalen
	-----------------------------------------	
	P_DEBUG : process(clk_in)
	begin
		if falling_edge(clk_in) then
			-------------------------------------------		
			case STATE_M is
				when M1_START_UP => v_main_status <= "00011000";
				when M2_WAIT_4_DONE => v_main_status <= "00100100";
				when M3_NOP => v_main_status <= "10000001";
				when M4_WRITE_INIT => v_main_status <= "00000001";
				when M5_WRITE => v_main_status <= "00000010";
				when M6_WRITING => v_main_status <= "00000100";
				when M7_WRITE_OK => v_main_status <= "00001000";
				when M8_READ_INIT => v_main_status <= "10000000";
				when M9_READ => v_main_status <= "01000000";
				when M10_READING => v_main_status <= "00100000";
				when M11_READ_OK => v_main_status <= "00010000";
				when others => v_main_status <= "10101010";
			end case;
			-------------------------------------------
			-- Main Status ausgeben
--			status_out <= v_main_status;			
			-------------------------------------------
			-- Write Status ausgeben
--			status_out <= v_write_status;
			-------------------------------------------
			-- Read Status ausgeben
--			status_out <= v_read_status;
			-------------------------------------------	
			-- Read Data ausgeben
			if btn_in(4)='0' and btn_in(5)='0' then
				status_out <= v_read_data(7 downto 0);
			elsif btn_in(4)='1' and btn_in(5)='0' then
				status_out <= v_read_data(15 downto 8);			
			elsif btn_in(4)='0' and btn_in(5)='1' then
				status_out <= v_read_data(23 downto 16);
			else 
				status_out <= v_read_data(31 downto 24);
			end if;
			-------------------------------------------			
		end if;
	end process P_DEBUG;


end Verhalten;

