----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:14:20 03/21/2011 
-- Design Name: 
-- Module Name:    DDR2_Write_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DDR2_Write_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		reset_in : in std_logic;
		clk_in : in std_logic;
		w_command_register : out std_logic_vector(2 downto 0);
		w_cmd_ack : in std_logic;
		w_burst_done : out std_logic;
		write_en : in std_logic;
		write_busy : out std_logic;
		write_status_out : out std_logic_vector(7 downto 0)
	);
	
end DDR2_Write_VHDL;

architecture Verhalten of DDR2_Write_VHDL is

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------
	
	constant clk_anz : integer := 2; -- warte 3 Clockzyklen (2 bis 0 = 3)
	
	signal v_counter :  natural range 0 to clk_anz := clk_anz;
	signal v_write_status_out : std_logic_vector(7 downto 0):= (others => '0');
	
	type STATE_W_TYPE is (
		W1_NOP,
		W2_WRITE_CMD,
		W3_WAIT_4_ACK1,
		W4_WAIT_3_CLK,
		W5_BURST_HI,
		W6_BURST_OK,
		W7_WAIT_4_ACK0
	);
	signal STATE_W : STATE_W_TYPE := W1_NOP;		

begin

	-----------------------------------------
	-- State-Machine zum
	-- schreiben eines wortes ins RAM
	-----------------------------------------	
	P_Write : process(clk_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_W <= W1_NOP;
			w_command_register <= "000"; -- NOP
			w_burst_done <= '0';
		elsif falling_edge(clk_in) then
			case STATE_W is
				when W1_NOP =>		
					-- warte auf write enable signal
					w_command_register <= "000"; -- NOP
					v_counter <= clk_anz;
					w_burst_done <= '0';
					if write_en = '1' then
						STATE_W <= W2_WRITE_CMD;						
					end if;								
				when W2_WRITE_CMD =>
					-- CMD anlegen
					w_command_register <= "100"; -- WRITE-CMD
					STATE_W <= W3_WAIT_4_ACK1;										
				when W3_WAIT_4_ACK1 =>
					-- warten auf ACK=1 vom RAM
					if w_cmd_ack = '1' then
						STATE_W <= W4_WAIT_3_CLK;
					end if;					
				when W4_WAIT_3_CLK =>
					-- warte 3 Clockzyklen
					if v_counter = 0 then
						-- burst_done auf HI
						w_burst_done <= '1';
						STATE_W <= W5_BURST_HI;						
					else
						w_burst_done <= '0';
						v_counter <= v_counter - 1;
					end if;	
				when W5_BURST_HI =>
					-- NOP anlegen
					w_command_register <= "000"; -- NOP
					STATE_W <= W6_BURST_OK;
				when W6_BURST_OK =>
					-- burst_done auf Lo
					w_burst_done <= '0';
					STATE_W <= W7_WAIT_4_ACK0;
				when W7_WAIT_4_ACK0 =>
					-- warten auf ACK=0 vom RAM
					if w_cmd_ack = '0' then
						STATE_W <= W1_NOP;
					end if;					
				when others =>
					NULL;
			end case;
		end if;
	end process P_Write;
	
	-----------------------------------------
	-- Write-Busy erzeugen
	-----------------------------------------		
	write_busy <= '0' when STATE_W=W1_NOP else '1';	
	
	-----------------------------------------
	-- Bearbeiten von Debug Signalen
	-----------------------------------------	
	P_DEBUG : process(clk_in)
	begin
		if falling_edge(clk_in) then
			case STATE_W is
				when W1_NOP => v_write_status_out <= "00111100";
				when W2_WRITE_CMD => v_write_status_out <= "00000001";
				when W3_WAIT_4_ACK1 => v_write_status_out <= "00000010";
				when W4_WAIT_3_CLK => v_write_status_out <= "00000100";
				when W5_BURST_HI => v_write_status_out <= "00001000";
				when W6_BURST_OK => v_write_status_out <= "00010000";
				when W7_WAIT_4_ACK0 => v_write_status_out <= "00100000";
				when others => v_write_status_out <= "01010101";
			end case;
		end if;
	end process P_DEBUG;
	
	write_status_out <= v_write_status_out;

end Verhalten;

